@@ -205,6 +205,7 @@ impl<'a> Resolved<'a> {
 struct ResolvedDiagnostic<'a> {
     level: AnnotateLevel,
     id: Option<String>,
+    documentation_url: Option<String>,
     message: String,
     annotations: Vec<ResolvedAnnotation<'a>>,
     is_fixable: bool,
@@ -240,12 +241,12 @@ impl<'a> ResolvedDiagnostic<'a> {
             // `DisplaySet::format_annotation` for both cases, but this is a small hack to improve
             // the formatting of syntax errors for now. This should also be kept consistent with the
             // concise formatting.
-            Some(diag.secondary_code().map_or_else(
+            diag.secondary_code().map_or_else(
                 || format!("{id}:", id = diag.inner.id),
                 |code| code.to_string(),
-            ))
+            )
         } else {
-            Some(diag.inner.id.to_string())
+            diag.inner.id.to_string()
         };
 
         let level = if config.hide_severity {
@@ -256,7 +257,8 @@ impl<'a> ResolvedDiagnostic<'a> {
 
         ResolvedDiagnostic {
             level,
-            id,
+            id: Some(id),
+            documentation_url: diag.documentation_url().map(ToString::to_string),
             message: diag.inner.message.as_str().to_string(),
             annotations,
             is_fixable: config.show_fix_status && diag.has_applicable_fix(config),
@@ -287,6 +289,7 @@ impl<'a> ResolvedDiagnostic<'a> {
         ResolvedDiagnostic {
             level: diag.inner.severity.to_annotate(),
             id: None,
+            documentation_url: None,
             message: diag.inner.message.as_str().to_string(),
             annotations,
             is_fixable: false,
@@ -385,6 +388,7 @@ impl<'a> ResolvedDiagnostic<'a> {
         RenderableDiagnostic {
             level: self.level,
             id: self.id.as_deref(),
+            documentation_url: self.documentation_url.as_deref(),
             message: &self.message,
             snippets_by_input,
             is_fixable: self.is_fixable,
@@ -485,6 +489,7 @@ struct RenderableDiagnostic<'r> {
     /// An ID is always present for top-level diagnostics and always absent for
     /// sub-diagnostics.
     id: Option<&'r str>,
+    documentation_url: Option<&'r str>,
     /// The message emitted with the diagnostic, before any snippets are
     /// rendered.
     message: &'r str,
@@ -519,7 +524,7 @@ impl RenderableDiagnostic<'_> {
             .is_fixable(self.is_fixable)
             .lineno_offset(self.header_offset);
         if let Some(id) = self.id {
-            message = message.id(id);
+            message = message.id_with_url(id, self.documentation_url);
         }
         message.snippets(snippets)
     }
@@ -1082,7 +1087,8 @@ impl<'r> EscapedSourceCode<'r> {
                 continue;
             }
             let start = range.start();
-            let end = ceil_char_boundary(&self.text, start + TextSize::from(1));
+            let end =
+                TextSize::try_from(self.text.ceil_char_boundary(start.to_usize() + 1)).unwrap();
             ann.range = TextRange::new(start, end);
         }
 
@@ -1090,86 +1096,6 @@ impl<'r> EscapedSourceCode<'r> {
     }
 }
 
-/// Finds the closest [`TextSize`] not less than the offset given for which
-/// `is_char_boundary` is `true`. Unless the offset given is greater than
-/// the length of the underlying contents, in which case, the length of the
-/// contents is returned.
-///
-/// Can be replaced with `str::ceil_char_boundary` once it's stable.
-///
-/// # Examples
-///
-/// From `std`:
-///
-/// ```
-/// use ruff_db::diagnostic::ceil_char_boundary;
-/// use ruff_text_size::{Ranged, TextLen, TextSize};
-///
-/// let source = "❤️🧡💛💚💙💜";
-/// assert_eq!(source.text_len(), TextSize::from(26));
-/// assert!(!source.is_char_boundary(13));
-///
-/// let closest = ceil_char_boundary(source, TextSize::from(13));
-/// assert_eq!(closest, TextSize::from(14));
-/// assert_eq!(&source[..closest.to_usize()], "❤️🧡💛");
-/// ```
-///
-/// Additional examples:
-///
-/// ```
-/// use ruff_db::diagnostic::ceil_char_boundary;
-/// use ruff_text_size::{Ranged, TextRange, TextSize};
-///
-/// let source = "Hello";
-///
-/// assert_eq!(
-///     ceil_char_boundary(source, TextSize::from(0)),
-///     TextSize::from(0)
-/// );
-///
-/// assert_eq!(
-///     ceil_char_boundary(source, TextSize::from(5)),
-///     TextSize::from(5)
-/// );
-///
-/// assert_eq!(
-///     ceil_char_boundary(source, TextSize::from(6)),
-///     TextSize::from(5)
-/// );
-///
-/// let source = "α";
-///
-/// assert_eq!(
-///     ceil_char_boundary(source, TextSize::from(0)),
-///     TextSize::from(0)
-/// );
-///
-/// assert_eq!(
-///     ceil_char_boundary(source, TextSize::from(1)),
-///     TextSize::from(2)
-/// );
-///
-/// assert_eq!(
-///     ceil_char_boundary(source, TextSize::from(2)),
-///     TextSize::from(2)
-/// );
-///
-/// assert_eq!(
-///     ceil_char_boundary(source, TextSize::from(3)),
-///     TextSize::from(2)
-/// );
-/// ```
-pub fn ceil_char_boundary(text: &str, offset: TextSize) -> TextSize {
-    let upper_bound = offset
-        .to_u32()
-        .saturating_add(4)
-        .min(text.text_len().to_u32());
-    (offset.to_u32()..upper_bound)
-        .map(TextSize::from)
-        .find(|offset| text.is_char_boundary(offset.to_usize()))
-        .unwrap_or_else(|| TextSize::from(upper_bound))
-}
-
 /// A stub implementation of [`FileResolver`] intended for testing.
 pub struct DummyFileResolver;
 
@@ -1279,7 +1205,7 @@ watermelon
         let diag = env.err().primary("animals", "5", "5", "").build();
         insta::assert_snapshot!(
             env.render(&diag),
-            @r"
+            @"
         error[test-diagnostic]: main diagnostic message
          --> animals:5:1
           |
@@ -1303,7 +1229,7 @@ watermelon
             .build();
         insta::assert_snapshot!(
             env.render(&diag),
-            @r"
+            @"
         warning[test-diagnostic]: main diagnostic message
          --> animals:5:1
           |
@@ -1323,7 +1249,7 @@ watermelon
             .build();
         insta::assert_snapshot!(
             env.render(&diag),
-            @r"
+            @"
         info[test-diagnostic]: main diagnostic message
          --> animals:5:1
           |
@@ -1350,7 +1276,7 @@ watermelon
         let diag = builder.build();
         insta::assert_snapshot!(
             env.render(&diag),
-            @r"
+            @"
         error[test-diagnostic]: main diagnostic message
          --> animals:1:1
           |
@@ -1369,7 +1295,7 @@ watermelon
         let diag = builder.build();
         insta::assert_snapshot!(
             env.render(&diag),
-            @r"
+            @"
         error[test-diagnostic]: main diagnostic message
          --> animals:1:1
           |
@@ -1390,7 +1316,7 @@ watermelon
         let diag = env.err().primary("non-ascii", "5", "5", "").build();
         insta::assert_snapshot!(
             env.render(&diag),
-            @r"
+            @"
         error[test-diagnostic]: main diagnostic message
          --> non-ascii:5:1
           |
@@ -1409,7 +1335,7 @@ watermelon
         let diag = env.err().primary("non-ascii", "2:4", "2:8", "").build();
         insta::assert_snapshot!(
             env.render(&diag),
-            @r"
+            @"
         error[test-diagnostic]: main diagnostic message
          --> non-ascii:2:2
           |
@@ -1433,7 +1359,7 @@ watermelon
         env.context(1);
         insta::assert_snapshot!(
             env.render(&diag),
-            @r"
+            @"
         error[test-diagnostic]: main diagnostic message
          --> animals:5:1
           |
@@ -1450,7 +1376,7 @@ watermelon
         env.context(0);
         insta::assert_snapshot!(
             env.render(&diag),
-            @r"
+            @"
         error[test-diagnostic]: main diagnostic message
          --> animals:5:1
           |
@@ -1465,7 +1391,7 @@ watermelon
         env.context(2);
         insta::assert_snapshot!(
             env.render(&diag),
-            @r"
+            @"
         error[test-diagnostic]: main diagnostic message
          --> animals:1:1
           |
@@ -1482,7 +1408,7 @@ watermelon
         env.context(2);
         insta::assert_snapshot!(
             env.render(&diag),
-            @r"
+            @"
         error[test-diagnostic]: main diagnostic message
           --> animals:11:1
            |
@@ -1499,7 +1425,7 @@ watermelon
         env.context(200);
         insta::assert_snapshot!(
             env.render(&diag),
-            @r"
+            @"
         error[test-diagnostic]: main diagnostic message
           --> animals:5:1
            |
@@ -1532,7 +1458,7 @@ watermelon
             .build();
         insta::assert_snapshot!(
             env.render(&diag),
-            @r"
+            @"
         error[test-diagnostic]: main diagnostic message
           --> animals:1:1
            |
@@ -1576,7 +1502,7 @@ watermelon
             .build();
         insta::assert_snapshot!(
             env.render(&diag),
-            @r"
+            @"
         error[test-diagnostic]: main diagnostic message
          --> animals:1:1
           |
@@ -1601,7 +1527,7 @@ watermelon
             .build();
         insta::assert_snapshot!(
             env.render(&diag),
-            @r"
+            @"
         error[test-diagnostic]: main diagnostic message
          --> animals:1:1
           |
@@ -1629,7 +1555,7 @@ watermelon
             .build();
         insta::assert_snapshot!(
             env.render(&diag),
-            @r"
+            @"
         error[test-diagnostic]: main diagnostic message
          --> animals:1:1
           |
@@ -1657,7 +1583,7 @@ watermelon
             .build();
         insta::assert_snapshot!(
             env.render(&diag),
-            @r"
+            @"
         error[test-diagnostic]: main diagnostic message
          --> animals:1:1
           |
@@ -1682,7 +1608,7 @@ watermelon
             .build();
         insta::assert_snapshot!(
             env.render(&diag),
-            @r"
+            @"
         error[test-diagnostic]: main diagnostic message
           --> animals:1:1
            |
@@ -1713,7 +1639,7 @@ watermelon
         // window.
         insta::assert_snapshot!(
             env.render(&diag),
-            @r"
+            @"
         error[test-diagnostic]: main diagnostic message
           --> animals:1:1
            |
@@ -1751,7 +1677,7 @@ watermelon
         let diag = env.err().primary("spacey-animals", "8", "8", "").build();
         insta::assert_snapshot!(
             env.render(&diag),
-            @r"
+            @"
         error[test-diagnostic]: main diagnostic message
          --> spacey-animals:8:1
           |
@@ -1768,7 +1694,7 @@ watermelon
         let diag = env.err().primary("spacey-animals", "12", "12", "").build();
         insta::assert_snapshot!(
             env.render(&diag),
-            @r"
+            @"
         error[test-diagnostic]: main diagnostic message
           --> spacey-animals:12:1
            |
@@ -1786,7 +1712,7 @@ watermelon
         let diag = env.err().primary("spacey-animals", "13", "13", "").build();
         insta::assert_snapshot!(
             env.render(&diag),
-            @r"
+            @"
         error[test-diagnostic]: main diagnostic message
           --> spacey-animals:13:1
            |
@@ -1826,7 +1752,7 @@ watermelon
         // instead of special casing the snippet assembly.
         insta::assert_snapshot!(
             env.render(&diag),
-            @r"
+            @"
         error[test-diagnostic]: main diagnostic message
          --> spacey-animals:3:1
           |
@@ -1855,7 +1781,7 @@ watermelon
             .build();
         insta::assert_snapshot!(
             env.render(&diag),
-            @r"
+            @"
         error[test-diagnostic]: main diagnostic message
          --> animals:3:1
           |
@@ -1892,7 +1818,7 @@ watermelon
         );
         insta::assert_snapshot!(
             env.render(&diag),
-            @r"
+            @"
         error[test-diagnostic]: main diagnostic message
          --> animals:3:1
           |
@@ -1929,7 +1855,7 @@ watermelon
         );
         insta::assert_snapshot!(
             env.render(&diag),
-            @r"
+            @"
         error[test-diagnostic]: main diagnostic message
          --> animals:3:1
           |
@@ -1957,7 +1883,7 @@ watermelon
         diag.sub(env.sub_warn().primary("fruits", "3", "3", "").build());
         insta::assert_snapshot!(
             env.render(&diag),
-            @r"
+            @"
         error[test-diagnostic]: main diagnostic message
          --> animals:3:1
           |
@@ -1993,7 +1919,7 @@ watermelon
         diag.sub(env.sub_warn().primary("animals", "11", "11", "").build());
         insta::assert_snapshot!(
             env.render(&diag),
-            @r"
+            @"
         error[test-diagnostic]: main diagnostic message
          --> animals:3:1
           |
@@ -2032,7 +1958,7 @@ watermelon
         diag.sub(env.sub_warn().primary("fruits", "3", "3", "").build());
         insta::assert_snapshot!(
             env.render(&diag),
-            @r"
+            @"
         error[test-diagnostic]: main diagnostic message
          --> animals:3:1
           |
@@ -2080,7 +2006,7 @@ watermelon
         diag.sub(env.sub_warn().secondary("animals", "3", "3", "").build());
         insta::assert_snapshot!(
             env.render(&diag),
-            @r"
+            @"
         error[test-diagnostic]: main diagnostic message
          --> animals:3:1
           |
@@ -2116,7 +2042,7 @@ watermelon
         let diag = env.err().primary("animals", "5", "6", "").build();
         insta::assert_snapshot!(
             env.render(&diag),
-            @r"
+            @"
         error[test-diagnostic]: main diagnostic message
          --> animals:5:1
           |
@@ -2139,7 +2065,7 @@ watermelon
         let diag = env.err().primary("animals", "5", "7:0", "").build();
         insta::assert_snapshot!(
             env.render(&diag),
-            @r"
+            @"
         error[test-diagnostic]: main diagnostic message
          --> animals:5:1
           |
@@ -2159,7 +2085,7 @@ watermelon
         let diag = env.err().primary("animals", "5", "7:1", "").build();
         insta::assert_snapshot!(
             env.render(&diag),
-            @r"
+            @"
         error[test-diagnostic]: main diagnostic message
          --> animals:5:1
           |
@@ -2179,7 +2105,7 @@ watermelon
         let diag = env.err().primary("animals", "5:3", "8:8", "").build();
         insta::assert_snapshot!(
             env.render(&diag),
-            @r"
+            @"
         error[test-diagnostic]: main diagnostic message
           --> animals:5:4
            |
@@ -2201,7 +2127,7 @@ watermelon
         let diag = env.err().secondary("animals", "5:3", "8:8", "").build();
         insta::assert_snapshot!(
             env.render(&diag),
-            @r"
+            @"
         error[test-diagnostic]: main diagnostic message
           --> animals:5:4
            |
@@ -2233,7 +2159,7 @@ watermelon
             .build();
         insta::assert_snapshot!(
             env.render(&diag),
-            @r"
+            @"
         error[test-diagnostic]: main diagnostic message
          --> animals:4:1
           |
@@ -2262,7 +2188,7 @@ watermelon
             .build();
         insta::assert_snapshot!(
             env.render(&diag),
-            @r"
+            @"
         error[test-diagnostic]: main diagnostic message
          --> animals:4:1
           |
@@ -2293,7 +2219,7 @@ watermelon
             .build();
         insta::assert_snapshot!(
             env.render(&diag),
-            @r"
+            @"
         error[test-diagnostic]: main diagnostic message
          --> animals:5:1
           |
@@ -2328,7 +2254,7 @@ watermelon
         // better using only ASCII art.
         insta::assert_snapshot!(
             env.render(&diag),
-            @r"
+            @"
         error[test-diagnostic]: main diagnostic message
          --> animals:5:1
           |
@@ -2356,7 +2282,7 @@ watermelon
             .build();
         insta::assert_snapshot!(
             env.render(&diag),
-            @r"
+            @"
         error[test-diagnostic]: main diagnostic message
          --> animals:5:1
           |
@@ -2388,7 +2314,7 @@ watermelon
             .build();
         insta::assert_snapshot!(
             env.render(&diag),
-            @r"
+            @"
         error[test-diagnostic]: main diagnostic message
          --> animals:5:3
           |
@@ -2410,7 +2336,7 @@ watermelon
             .build();
         insta::assert_snapshot!(
             env.render(&diag),
-            @r"
+            @"
         error[test-diagnostic]: main diagnostic message
          --> animals:5:3
           |
@@ -2443,7 +2369,7 @@ watermelon
             .build();
         insta::assert_snapshot!(
             env.render(&diag),
-            @r"
+            @"
         error[test-diagnostic]: main diagnostic message
           --> animals:8:1
            |
@@ -2483,7 +2409,7 @@ watermelon
             .build();
         insta::assert_snapshot!(
             env.render(&diag),
-            @r"
+            @"
         error[test-diagnostic]: main diagnostic message
          --> animals:5:1
           |
@@ -2527,7 +2453,7 @@ watermelon
             .build();
         insta::assert_snapshot!(
             env.render(&diag),
-            @r"
+            @"
         error[test-diagnostic]: main diagnostic message
          --> fruits:1:1
           |
@@ -2562,7 +2488,7 @@ watermelon
             .build();
         insta::assert_snapshot!(
             env.render(&diag),
-            @r"
+            @"
         error[test-diagnostic]: main diagnostic message
           --> animals:11:1
            |
@@ -2876,6 +2802,12 @@ watermelon
             self.diag.help(message);
             self
         }
+
+        /// Set the documentation URL for the diagnostic.
+        pub(super) fn documentation_url(mut self, url: impl Into<String>) -> DiagnosticBuilder<'e> {
+            self.diag.set_documentation_url(Some(url.into()));
+            self
+        }
     }
 
     /// A helper builder for tersely populating a `SubDiagnostic`.
@@ -2990,6 +2922,7 @@ def fibonacci(n):
                     TextSize::from(10),
                 ))))
                 .noqa_offset(TextSize::from(7))
+                .documentation_url("https://docs.astral.sh/ruff/rules/unused-import")
                 .build(),
             env.builder(
                 "unused-variable",
@@ -3004,11 +2937,13 @@ def fibonacci(n):
                 TextSize::from(99),
             )))
             .noqa_offset(TextSize::from(94))
+            .documentation_url("https://docs.astral.sh/ruff/rules/unused-variable")
             .build(),
             env.builder("undefined-name", Severity::Error, "Undefined name `a`")
                 .primary("undef.py", "1:3", "1:4", "")
                 .secondary_code("F821")
                 .noqa_offset(TextSize::from(3))
+                .documentation_url("https://docs.astral.sh/ruff/rules/undefined-name")
                 .build(),
         ];
 
@@ -3123,6 +3058,7 @@ if call(foo
                     TextSize::from(19),
                 ))))
                 .noqa_offset(TextSize::from(16))
+                .documentation_url("https://docs.astral.sh/ruff/rules/unused-import")
                 .build(),
             env.builder(
                 "unused-import",
@@ -3137,6 +3073,7 @@ if call(foo
                 TextSize::from(40),
             ))))
             .noqa_offset(TextSize::from(35))
+            .documentation_url("https://docs.astral.sh/ruff/rules/unused-import")
             .build(),
             env.builder(
                 "unused-variable",
@@ -3151,6 +3088,7 @@ if call(foo
                 TextSize::from(104),
             ))))
             .noqa_offset(TextSize::from(98))
+            .documentation_url("https://docs.astral.sh/ruff/rules/unused-variable")
             .build(),
         ];
 
